<?php
/*
 *  $Id$
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many individuals
 * and is licensed under the LGPL. For more information, see
 * <http://www.doctrine-project.org>.
 */
 
namespace Doctrine\ORM\Tools\Cli\Tasks;

use Doctrine\Common\Cli\Tasks\AbstractTask,
    Doctrine\Common\Cli\Option,
    Doctrine\Common\Cli\OptionGroup,
    Doctrine\Common\Cli\CliException,
    Doctrine\ORM\Tools\ClassMetadataReader;

/**
 * CLI Task to generate repository classes for some mapping information
 *
 * @license http://www.opensource.org/licenses/lgpl-license.php LGPL
 * @link    www.doctrine-project.org
 * @since   2.0
 * @version $Revision$
 * @author  Benjamin Eberlei <kontakt@beberlei.de>
 * @author  Guilherme Blanco <guilhermeblanco@hotmail.com>
 * @author  Jonathan Wage <jonwage@gmail.com>
 * @author  Roman Borschel <roman@code-factory.org>
 */
class GenerateRepositoriesTask extends ConvertMappingTask
{
    private static $_template =
'<?php

namespace <namespace>;

use \Doctrine\ORM\EntityRepository;

/**
 * <className>
 *
 * This class was generated by the Doctrine ORM. Add your own custom
 * repository methods below.
 */
class <className> extends EntityRepository
{
}';

    /**
     * @inheritdoc
     */
    public function buildDocumentation()
    {
        $options = new OptionGroup(OptionGroup::CARDINALITY_N_N, array(
            new Option('from', '<FROM>', 'The path to mapping information.'),
            new Option('dest', '<DEST>', 'The path to generate your repository classes.')
        ));

        $doc = $this->getDocumentation();
        $doc->setName('generate-repositories')
            ->setDescription('Generate repository classes for some mapping information.')
            ->getOptionGroup()
                ->addOption($options);
    }

    /**
     * @inheritdoc
     */
    public function validate()
    {
        $arguments = $this->getArguments();
        $em = $this->getConfiguration()->getAttribute('em');

        if ( ! isset($arguments['from']) || ! isset($arguments['dest'])) {
            throw new CliException('You must specify a value for --from and --dest');
        }

        return true;
    }

    public function run()
    {
        $printer = $this->getPrinter();
        $arguments = $this->getArguments();
        $dest = realpath($arguments['dest']);

        $reader = new ClassMetadataReader();
        $reader->setEntityManager($this->getConfiguration()->getAttribute('em'));
        $reader->addMappingSource($arguments['from']);
        $metadatas = $reader->getMetadatas();

        foreach ($metadatas as $metadata) {
            if ($metadata->customRepositoryClassName) {
                $code = $this->_generateRepositoryClass($metadata->customRepositoryClassName);
                $path = $dest . '/' . str_replace('\\', '/', $metadata->customRepositoryClassName) . '.php';
                $dir = dirname($path);
                if ( ! is_dir($dir)) {
                    mkdir($dir, 0777, true);
                }
                $printer->writeln(
                    sprintf('Processing entity "%s"',
                        $printer->format($metadata->customRepositoryClassName, 'KEYWORD')
                    )
                );
                file_put_contents($path, $code);
            }
        }
        $printer->writeln('');
        $printer->writeln(
            sprintf('Entity repository classes generated to "%s"',
                $printer->format($dest, 'KEYWORD')
            )
        );
    }

    private function _generateRepositoryClass($fullyQualifiedClassName)
    {
        $namespace = substr($fullyQualifiedClassName, 0, strrpos($fullyQualifiedClassName, '\\'));

        $pos = strrpos($fullyQualifiedClassName, '\\');
        $className = substr($fullyQualifiedClassName, $pos + 1, strlen($fullyQualifiedClassName));

        $placeHolders = array(
            '<namespace>',
            '<className>'
        );
        $replacements = array(
            $namespace,
            $className
        );
        $code = str_replace($placeHolders, $replacements, self::$_template);
        return $code;
    }
}